"""Database model definitions and table mappings for the GSO system."""

import structlog
from orchestrator.db import UtcTimestamp
from orchestrator.db.database import BaseModel
from sqlalchemy import (
    String,
    text,
)
from sqlalchemy.orm import mapped_column

logger = structlog.get_logger(__name__)


class PartnerTable(BaseModel):
    """Database table for the partners in the GSO system."""

    __tablename__ = "partners"

    partner_id = mapped_column(String, server_default=text("uuid_generate_v4"), primary_key=True)
    name = mapped_column(String, unique=True, nullable=False)
    email = mapped_column(String, unique=True, nullable=False)

    created_at = mapped_column(UtcTimestamp, server_default=text("current_timestamp"), nullable=False)
    updated_at = mapped_column(
        UtcTimestamp, server_default=text("current_timestamp"), nullable=False, onupdate=text("current_timestamp")
    )
